/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kmenuedit.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "kmenueditadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class KmenueditAdaptor
 */

KmenueditAdaptor::KmenueditAdaptor(KMenuEdit *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

KmenueditAdaptor::~KmenueditAdaptor()
{
    // destructor
}

void KmenueditAdaptor::restoreSystemMenu()
{
    // handle method call org.kde.kmenuedit.restoreSystemMenu
    parent()->restoreSystemMenu();
}

void KmenueditAdaptor::selectMenu(const QString &menu)
{
    // handle method call org.kde.kmenuedit.selectMenu
    parent()->selectMenu(menu);
}

void KmenueditAdaptor::selectMenuEntry(const QString &menu)
{
    // handle method call org.kde.kmenuedit.selectMenuEntry
    parent()->selectMenuEntry(menu);
}


#include "moc_kmenueditadaptor.cpp"
