/****************************************************************************
** Meta object code from reading C++ file 'treeview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kmenuedit-6.6.4/treeview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'treeview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8TreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto TreeView::qt_create_metaobjectdata<qt_meta_tag_ZN8TreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreeView",
        "entrySelected",
        "",
        "MenuFolderInfo*",
        "folderInfo",
        "MenuEntryInfo*",
        "entryInfo",
        "disableAction",
        "currentDataChanged",
        "findServiceShortcut",
        "QKeySequence",
        "KService::Ptr&",
        "searchUpdated",
        "searchString",
        "itemSelected",
        "QTreeWidgetItem*",
        "dropMimeData",
        "parent",
        "index",
        "const QMimeData*",
        "data",
        "Qt::DropAction",
        "action",
        "newsubmenu",
        "newitem",
        "newsep",
        "copyFilePath",
        "openContainingFolder",
        "properties",
        "cut",
        "copy",
        "paste",
        "del",
        "sort",
        "sortCmd",
        "moveUpItem",
        "moveDownItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entrySelected'
        QtMocHelpers::SignalData<void(MenuFolderInfo *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'entrySelected'
        QtMocHelpers::SignalData<void(MenuEntryInfo *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'disableAction'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'currentDataChanged'
        QtMocHelpers::SlotData<void(MenuFolderInfo *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'currentDataChanged'
        QtMocHelpers::SlotData<void(MenuEntryInfo *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'findServiceShortcut'
        QtMocHelpers::SlotData<void(const QKeySequence &, KService::Ptr &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 }, { 0x80000000 | 11, 2 },
        }}),
        // Slot 'searchUpdated'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'itemSelected'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 2 },
        }}),
        // Slot 'dropMimeData'
        QtMocHelpers::SlotData<bool(QTreeWidgetItem *, int, const QMimeData *, Qt::DropAction)>(16, 2, QMC::AccessProtected, QMetaType::Bool, {{
            { 0x80000000 | 15, 17 }, { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
        }}),
        // Slot 'newsubmenu'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'newitem'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'newsep'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'copyFilePath'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openContainingFolder'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'properties'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'cut'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'copy'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'paste'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'del'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sort'
        QtMocHelpers::SlotData<void(const int)>(33, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'moveUpItem'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveDownItem'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreeView, qt_meta_tag_ZN8TreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8TreeViewE_t>.metaTypes,
    nullptr
} };

void TreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entrySelected((*reinterpret_cast<std::add_pointer_t<MenuFolderInfo*>>(_a[1]))); break;
        case 1: _t->entrySelected((*reinterpret_cast<std::add_pointer_t<MenuEntryInfo*>>(_a[1]))); break;
        case 2: _t->disableAction(); break;
        case 3: _t->currentDataChanged((*reinterpret_cast<std::add_pointer_t<MenuFolderInfo*>>(_a[1]))); break;
        case 4: _t->currentDataChanged((*reinterpret_cast<std::add_pointer_t<MenuEntryInfo*>>(_a[1]))); break;
        case 5: _t->findServiceShortcut((*reinterpret_cast<std::add_pointer_t<QKeySequence>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KService::Ptr&>>(_a[2]))); break;
        case 6: _t->searchUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->itemSelected((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 8: { bool _r = _t->dropMimeData((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<const QMimeData*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Qt::DropAction>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->newsubmenu(); break;
        case 10: _t->newitem(); break;
        case 11: _t->newsep(); break;
        case 12: _t->copyFilePath(); break;
        case 13: _t->openContainingFolder(); break;
        case 14: _t->properties(); break;
        case 15: _t->cut(); break;
        case 16: _t->copy(); break;
        case 17: _t->paste(); break;
        case 18: _t->del(); break;
        case 19: _t->sort((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->moveUpItem(); break;
        case 21: _t->moveDownItem(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreeView::*)(MenuFolderInfo * )>(_a, &TreeView::entrySelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeView::*)(MenuEntryInfo * )>(_a, &TreeView::entrySelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeView::*)()>(_a, &TreeView::disableAction, 2))
            return;
    }
}

const QMetaObject *TreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TreeViewE_t>.strings))
        return static_cast<void*>(this);
    return QTreeWidget::qt_metacast(_clname);
}

int TreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void TreeView::entrySelected(MenuFolderInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TreeView::entrySelected(MenuEntryInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TreeView::disableAction()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN16MenuItemMimeDataE_t {};
} // unnamed namespace

template <> constexpr inline auto MenuItemMimeData::qt_create_metaobjectdata<qt_meta_tag_ZN16MenuItemMimeDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MenuItemMimeData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MenuItemMimeData, qt_meta_tag_ZN16MenuItemMimeDataE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MenuItemMimeData::staticMetaObject = { {
    QMetaObject::SuperData::link<QMimeData::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MenuItemMimeDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MenuItemMimeDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16MenuItemMimeDataE_t>.metaTypes,
    nullptr
} };

void MenuItemMimeData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MenuItemMimeData *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *MenuItemMimeData::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MenuItemMimeData::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MenuItemMimeDataE_t>.strings))
        return static_cast<void*>(this);
    return QMimeData::qt_metacast(_clname);
}

int MenuItemMimeData::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMimeData::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
