/****************************************************************************
** Meta object code from reading C++ file 'basictab.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kmenuedit-6.6.4/basictab.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'basictab.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8BasicTabE_t {};
} // unnamed namespace

template <> constexpr inline auto BasicTab::qt_create_metaobjectdata<qt_meta_tag_ZN8BasicTabE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BasicTab",
        "changed",
        "",
        "MenuFolderInfo*",
        "MenuEntryInfo*",
        "findServiceShortcut",
        "QKeySequence",
        "KService::Ptr&",
        "setFolderInfo",
        "folderInfo",
        "setEntryInfo",
        "entryInfo",
        "slotDisableAction",
        "slotChanged",
        "launchcb_clicked",
        "termcb_clicked",
        "uidcb_clicked",
        "slotCapturedKeySequence",
        "slotExecSelected",
        "onlyshowcb_clicked",
        "hiddenentrycb_clicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void(MenuFolderInfo *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'changed'
        QtMocHelpers::SignalData<void(MenuEntryInfo *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'findServiceShortcut'
        QtMocHelpers::SignalData<void(const QKeySequence &, KService::Ptr &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { 0x80000000 | 7, 2 },
        }}),
        // Slot 'setFolderInfo'
        QtMocHelpers::SlotData<void(MenuFolderInfo *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 },
        }}),
        // Slot 'setEntryInfo'
        QtMocHelpers::SlotData<void(MenuEntryInfo *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 11 },
        }}),
        // Slot 'slotDisableAction'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'launchcb_clicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'termcb_clicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'uidcb_clicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCapturedKeySequence'
        QtMocHelpers::SlotData<void(const QKeySequence &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'slotExecSelected'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onlyshowcb_clicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'hiddenentrycb_clicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BasicTab, qt_meta_tag_ZN8BasicTabE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BasicTab::staticMetaObject = { {
    QMetaObject::SuperData::link<KPageWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8BasicTabE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8BasicTabE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8BasicTabE_t>.metaTypes,
    nullptr
} };

void BasicTab::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BasicTab *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed((*reinterpret_cast<std::add_pointer_t<MenuFolderInfo*>>(_a[1]))); break;
        case 1: _t->changed((*reinterpret_cast<std::add_pointer_t<MenuEntryInfo*>>(_a[1]))); break;
        case 2: _t->findServiceShortcut((*reinterpret_cast<std::add_pointer_t<QKeySequence>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KService::Ptr&>>(_a[2]))); break;
        case 3: _t->setFolderInfo((*reinterpret_cast<std::add_pointer_t<MenuFolderInfo*>>(_a[1]))); break;
        case 4: _t->setEntryInfo((*reinterpret_cast<std::add_pointer_t<MenuEntryInfo*>>(_a[1]))); break;
        case 5: _t->slotDisableAction(); break;
        case 6: _t->slotChanged(); break;
        case 7: _t->launchcb_clicked(); break;
        case 8: _t->termcb_clicked(); break;
        case 9: _t->uidcb_clicked(); break;
        case 10: _t->slotCapturedKeySequence((*reinterpret_cast<std::add_pointer_t<QKeySequence>>(_a[1]))); break;
        case 11: _t->slotExecSelected(); break;
        case 12: _t->onlyshowcb_clicked(); break;
        case 13: _t->hiddenentrycb_clicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BasicTab::*)(MenuFolderInfo * )>(_a, &BasicTab::changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BasicTab::*)(MenuEntryInfo * )>(_a, &BasicTab::changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BasicTab::*)(const QKeySequence & , KService::Ptr & )>(_a, &BasicTab::findServiceShortcut, 2))
            return;
    }
}

const QMetaObject *BasicTab::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BasicTab::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8BasicTabE_t>.strings))
        return static_cast<void*>(this);
    return KPageWidget::qt_metacast(_clname);
}

int BasicTab::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPageWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void BasicTab::changed(MenuFolderInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void BasicTab::changed(MenuEntryInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void BasicTab::findServiceShortcut(const QKeySequence & _t1, KService::Ptr & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
