/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // kmenueditui.rc
  0x0,0x0,0x1,0xba,
  0x28,
  0xb5,0x2f,0xfd,0x60,0x9,0x5,0x85,0xd,0x0,0x36,0xd1,0x36,0x1e,0x30,0x91,0x1e,
  0xf,0x49,0x22,0xdc,0x25,0xf8,0xd6,0x3c,0xfd,0xf1,0xc7,0xe6,0xcb,0x20,0x4b,0xb9,
  0x2f,0xf7,0xe0,0xb0,0x83,0x30,0x7b,0x50,0xd0,0x44,0xa9,0x31,0x0,0x2d,0x0,0x2c,
  0x0,0xff,0xaa,0xae,0xb1,0xa1,0x65,0xf,0xd,0x39,0x6e,0xf5,0x95,0xe0,0x9d,0xa7,
  0x65,0x99,0x14,0x45,0x26,0xc9,0xc4,0x5e,0xb5,0x58,0x4,0x91,0x44,0x16,0xf,0x6,
  0xe0,0x71,0x20,0xc4,0xe,0x4,0x84,0x3d,0x44,0xf9,0xa8,0x37,0x86,0x5d,0x50,0x83,
  0x78,0x2,0xce,0xb3,0xd9,0x4e,0x2a,0x6d,0xb,0xfb,0x9f,0xcb,0x74,0x8c,0xf2,0x3d,
  0x95,0x68,0x4f,0xc3,0x30,0x21,0x7,0xa0,0xd7,0xf4,0x6f,0x5b,0xde,0x92,0x8c,0x6c,
  0xfb,0x7a,0x14,0xc6,0x18,0x73,0x20,0x67,0x45,0x8d,0x2d,0x3d,0x4e,0x61,0x3,0x19,
  0x59,0xb5,0x9d,0xd5,0x35,0x46,0xc2,0xe8,0x5f,0xbd,0x34,0xda,0xef,0xdd,0xa7,0x77,
  0x75,0x49,0x8a,0xb3,0xe2,0x8b,0x8f,0x3f,0xc6,0x9e,0xb2,0x6d,0xe5,0x79,0x6e,0xa6,
  0xa4,0x18,0x65,0xb7,0x8d,0x61,0xea,0x26,0x97,0xcf,0x1,0xe,0xf4,0xaa,0xd5,0x9a,
  0xe2,0x39,0x31,0x87,0xb7,0xe5,0x75,0x2,0xb9,0xd2,0x1e,0xe7,0x9,0x93,0xe2,0x63,
  0x78,0x49,0x19,0x79,0x1c,0x7,0x37,0xd4,0x35,0x6d,0x7f,0x97,0xb4,0xaf,0xae,0xfa,
  0x29,0xcf,0xfd,0x4a,0xb9,0x8d,0x1,0x5a,0xa0,0xe1,0xa9,0x32,0x56,0x14,0x24,0x29,
  0x14,0xda,0x33,0x30,0x42,0x62,0x2c,0xb2,0xdb,0xb2,0x27,0xd2,0xa,0xb,0x25,0x41,
  0xe3,0x81,0x64,0x6a,0xf1,0x9a,0xae,0x7f,0xc,0x76,0x28,0x42,0x51,0x8b,0x99,0xae,
  0x47,0x26,0x99,0xda,0x1b,0xa0,0xc7,0xf0,0x11,0xd3,0x23,0x92,0x73,0xb7,0x9f,0xf5,
  0xb7,0x3c,0x71,0x66,0x46,0x4,0x3a,0x3,0xcb,0xce,0x3c,0x93,0x7f,0x55,0xe,0xb2,
  0x66,0x7,0x14,0x90,0xd2,0xb4,0xfa,0x3e,0x72,0xd9,0x5,0x72,0xa,0x80,0x42,0xc0,
  0xd8,0x91,0x8a,0x17,0x11,0x1,0xa2,0x8d,0xfd,0xb4,0x60,0x8a,0x22,0x49,0xe3,0x43,
  0xe1,0xd2,0x24,0x7b,0x27,0x22,0xd,0xe0,0xc6,0x37,0x2b,0x4,0xb2,0xd3,0x71,0xd1,
  0x8,0x4f,0x6a,0xb4,0xbd,0x5,0x1,0x1d,0xeb,0x1,0x38,0xe,0xf9,0x58,0x80,0xcc,
  0xc,0x9,0x21,0x94,0xf0,0x82,0x48,0x63,0x26,0x28,0x34,0x6,0x30,0x20,0xb8,0x10,
  0xf9,0x1,0xdd,0x21,0x28,0x81,0x57,0x83,0x3d,0xdf,0x97,0xa0,0x5b,0x1,0xb0,0x50,
  0x88,0x7b,0x2d,0x63,0x52,0xc6,0xc9,0x35,0x2f,0x63,0x7e,0x29,0x13,0x6b,0x38,0x3,
  0x31,0x1,0xd5,0xf7,0xe6,0x0,0xca,0x81,0x6b,0x99,0xc5,0xf6,0x7,0x41,0xcd,0x39,
  0xf3,0x3,0x40,0x8,0xf0,0x41,0x0,0xaf,0x2,
  
};

static const unsigned char qt_resource_name[] = {
  // kxmlgui5
  0x0,0x8,
  0xf,0x42,0xe5,0x85,
  0x0,0x6b,
  0x0,0x78,0x0,0x6d,0x0,0x6c,0x0,0x67,0x0,0x75,0x0,0x69,0x0,0x35,
    // kmenuedit
  0x0,0x9,
  0xc,0x5b,0x5f,0x64,
  0x0,0x6b,
  0x0,0x6d,0x0,0x65,0x0,0x6e,0x0,0x75,0x0,0x65,0x0,0x64,0x0,0x69,0x0,0x74,
    // kmenueditui.rc
  0x0,0xe,
  0x7,0x30,0xee,0x63,
  0x0,0x6b,
  0x0,0x6d,0x0,0x65,0x0,0x6e,0x0,0x75,0x0,0x65,0x0,0x64,0x0,0x69,0x0,0x74,0x0,0x75,0x0,0x69,0x0,0x2e,0x0,0x72,0x0,0x63,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9d,0x67,0xeb,0x7f,0x20,
  // :/kxmlgui5
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9d,0x67,0xeb,0x7f,0x20,
  // :/kxmlgui5/kmenuedit
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9d,0x67,0xeb,0x7f,0x20,
  // :/kxmlgui5/kmenuedit/kmenueditui.rc
  0x0,0x0,0x0,0x2e,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9d,0x67,0xeb,0x7f,0x20,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_kmenuedit)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_kmenuedit)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kmenuedit)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kmenuedit)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_kmenuedit)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kmenuedit)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
